package com.sun.xml.internal.ws.client.sei;

import com.sun.xml.internal.ws.api.message.Headers;
import com.sun.xml.internal.ws.spi.db.XMLBridge;
import com.sun.xml.internal.ws.message.JAXBAttachment;
import com.sun.xml.internal.ws.message.DataHandlerAttachment;
import com.sun.xml.internal.ws.api.message.Attachment;
import com.sun.xml.internal.ws.message.ByteArrayAttachment;
import java.util.UUID;
import javax.xml.transform.Source;
import javax.activation.DataHandler;
import java.io.UnsupportedEncodingException;
import javax.xml.ws.WebServiceException;
import java.net.URLEncoder;
import com.sun.xml.internal.ws.model.ParameterImpl;
import com.sun.xml.internal.ws.api.message.Message;

abstract class MessageFiller
{
    protected final int methodPos;
    
    protected MessageFiller(int methodPos) {
        this.methodPos = methodPos;
    }
    
    abstract void fillIn(Object[] p0, Message p1);
    
    private static boolean isXMLMimeType(String mimeType) {
        return mimeType.equals("text/xml") || mimeType.equals("application/xml");
    }
    
    static boolean access$000(String x0) {
        return isXMLMimeType(x0);
    }
    
    abstract static class AttachmentFiller extends MessageFiller
    {
        protected final ParameterImpl param;
        protected final ValueGetter getter;
        protected final String mimeType;
        private final String contentIdPart;
        
        protected AttachmentFiller(ParameterImpl param, ValueGetter getter) {
            super(param.getIndex());
            this.param = param;
            this.getter = getter;
            this.mimeType = param.getBinding().getMimeType();
            try {
                this.contentIdPart = URLEncoder.encode(param.getPartName(), "UTF-8") + 61;
            }
            catch (UnsupportedEncodingException e) {
                throw new WebServiceException(e);
            }
        }
        
        public static MessageFiller createAttachmentFiller(ParameterImpl param, ValueGetter getter) {
            Class<Object> type = (Class)param.getTypeInfo().type;
            if (DataHandler.class.isAssignableFrom(type) || Source.class.isAssignableFrom(type)) {
                return new MessageFiller.DataHandlerFiller(param, getter);
            }
            if (byte[].class == type) {
                return new MessageFiller.ByteArrayFiller(param, getter);
            }
            if (MessageFiller.access$000(param.getBinding().getMimeType())) {
                return new MessageFiller.JAXBFiller(param, getter);
            }
            return new MessageFiller.DataHandlerFiller(param, getter);
        }
        
        String getContentId() {
            return this.contentIdPart + UUID.randomUUID() + "@jaxws.sun.com";
        }
    }
    
    private static class ByteArrayFiller extends MessageFiller.AttachmentFiller
    {
        protected ByteArrayFiller(ParameterImpl param, ValueGetter getter) {
            super(param, getter);
        }
        
        @Override
        void fillIn(Object[] methodArgs, Message msg) {
            String contentId = this.getContentId();
            Object obj = this.getter.get(methodArgs[this.methodPos]);
            Attachment att = new ByteArrayAttachment(contentId, (byte[])(byte[])obj, this.mimeType);
            msg.getAttachments().add(att);
        }
    }
    
    private static class DataHandlerFiller extends MessageFiller.AttachmentFiller
    {
        protected DataHandlerFiller(ParameterImpl param, ValueGetter getter) {
            super(param, getter);
        }
        
        @Override
        void fillIn(Object[] methodArgs, Message msg) {
            String contentId = this.getContentId();
            Object obj = this.getter.get(methodArgs[this.methodPos]);
            DataHandler dh = (obj instanceof DataHandler) ? ((DataHandler)obj) : new DataHandler(obj, this.mimeType);
            Attachment att = new DataHandlerAttachment(contentId, dh);
            msg.getAttachments().add(att);
        }
    }
    
    private static class JAXBFiller extends MessageFiller.AttachmentFiller
    {
        protected JAXBFiller(ParameterImpl param, ValueGetter getter) {
            super(param, getter);
        }
        
        @Override
        void fillIn(Object[] methodArgs, Message msg) {
            String contentId = this.getContentId();
            Object obj = this.getter.get(methodArgs[this.methodPos]);
            Attachment att = new JAXBAttachment(contentId, obj, this.param.getXMLBridge(), this.mimeType);
            msg.getAttachments().add(att);
        }
    }
    
    static final class Header extends MessageFiller
    {
        private final XMLBridge bridge;
        private final ValueGetter getter;
        
        protected Header(int methodPos, XMLBridge bridge, ValueGetter getter) {
            super(methodPos);
            this.bridge = bridge;
            this.getter = getter;
        }
        
        @Override
        void fillIn(Object[] methodArgs, Message msg) {
            Object value = this.getter.get(methodArgs[this.methodPos]);
            msg.getHeaders().add(Headers.create(this.bridge, value));
        }
    }
}
